<?php

namespace Drupal\commerce_shipping_product_exclude\Plugin\Commerce\Condition;

use Drupal\commerce\Plugin\Commerce\Condition\ConditionBase;
use Drupal\commerce_product\Entity\ProductVariationInterface;
use Drupal\commerce_shipping_product_exclude\Plugin\Field\FieldType\ExcludeShippingMethodItem;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Form\FormStateInterface;

/**
 * Provides the weight condition for shipments.
 *
 * @CommerceCondition(
 *   id = "exclude_from_shipping",
 *   label = @Translation("Allow to exclude From Shipping"),
 *   description = @Translation("Allows the targeted shipping method, to not appear, if ANY product's / product variation's 'Exclude Shipping Method' field in the order references it."),
 *   category = @Translation("Commerce Order"),
 *   entity_type = "commerce_order",
 * )
 */
class ExcludeFromShipping extends ConditionBase {

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'commerce_shipping_method_id' => NULL,
    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);
    // @todo We are currently using the configuration as a way to save the
    // currently targeted shipping method, as there is no way to access the
    // this entity otherwise.
    // This can be removed in the future, as we can simply get the
    // "target_entity" in the "evaluate" method, when that is implemented.
    // @see https://www.drupal.org/project/commerce/issues/3505575
    $this->configuration['commerce_shipping_method_id'] = $form_state->getFormObject()?->getEntity()?->id();
    if (empty($this->configuration['commerce_shipping_method_id'])) {
      $this->messenger()->addWarning($this->t('Please resave the shipping method "%method", so that the "%condition" condition can apply properly (see <a href=":link" target="_blank">here</a> for more information).', [
        '%condition' => $this->getLabel(),
        '%method' => reset($form_state->getValue('name'))['value'],
        ':link' => 'https://www.drupal.org/project/commerce/issues/3505575',
      ]));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function evaluate(EntityInterface $entity) {
    $this->assertEntity($entity);
    /** @var \Drupal\commerce_order\Entity\OrderInterface $order */
    $order = $entity;
    // Get the current shipping method from the configuration:
    $currentShippingMethodId = $this->configuration['commerce_shipping_method_id'];

    foreach ($order->getItems() as $orderItem) {
      $productVariation = $orderItem->getPurchasedEntity();
      if (!($productVariation instanceof ProductVariationInterface)) {
        // For some reason, the purchased entity is not a product variation.
        // Skip it:
        continue;
      }

      // Check if the product variation has an 'exclude_shipping_method' field,
      // which isn't empty:
      foreach ($productVariation->getFields() as $field) {
        if ($field->getFieldDefinition()->getType() === ExcludeShippingMethodItem::FIELD_ID) {
          if (!empty($field->getValue())) {
            // Check if the current shipping method should be excluded on
            // variation level:
            $excludedShippingMethods = $field->getValue();
            foreach ($excludedShippingMethods as $excludedShippingMethod) {
              if ($excludedShippingMethod['target_id'] === $currentShippingMethodId) {
                // The shipping method was found and should be excluded, the
                // evaluation fails:
                return FALSE;
              }
            }
          }
        }
      }

      // If the exclude shipping field is still empty, we check if the parent
      // product has the field:
      $product = $productVariation->getProduct();
      foreach ($product->getFields() as $field) {
        if ($field->getFieldDefinition()->getType() === ExcludeShippingMethodItem::FIELD_ID) {
          if (!empty($field->getValue())) {
            // Check if the current shipping method should be excluded on
            // product level:
            $excludedShippingMethods = $field->getValue();
            foreach ($excludedShippingMethods as $excludedShippingMethod) {
              if ($excludedShippingMethod['target_id'] === $currentShippingMethodId) {
                // The shipping method was found and should be excluded, the
                // evaluation fails:
                return FALSE;
              }
            }
          }
        }
      }
    }
    // This shipping method is not excluded, let the evaluation succeed:
    return TRUE;
  }

}
